/***************************************************************************
                          ksubeditor.cpp  -  description
                             -------------------
    begin                : sob sie  3 13:14:37 CEST 2002
    copyright            : (C) 2002 by Anatol Ogrek
    email                : mroova@linux.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#define READ_ERROR    -2
#define SUB_INVALID   -1
#define SUB_MICRODVD  0
#define SUB_SUBRIP    1
#define SUB_SUBVIEWER 2
#define SUB_SAMI      3
#define SUB_VPLAYER   4
#define SUB_RT        5
#define SUB_SSA       6
#define SUB_DUNNOWHAT 7		// FIXME what format is it ?
#define SUB_MPSUB     8
#define SUB_AQTITLE   9
#define SUB_MAX_TEXT  3
#define LINE_LEN  1000
#define ID_PLAY 14000
#define ID_MOVE_TIME 14010
#define ID_CHARSET 14020
// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qvbox.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qtextstream.h>
#include <qtextcodec.h>
#include <qstring.h>
#include <qspinbox.h>
#include <klistview.h>
#include <qradiobutton.h>
// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>
#include <ktempfile.h>
#include <kaccel.h>
#include <kkeydialog.h>
#include <kurlrequester.h>
#include <qmultilineedit.h>
// application specific includes
#include "ksubeditor.h"
#include "ksubeditorview.h"
#include "ksubeditordoc.h"
#include "configure.h"
#include "configuration.h"
#include <stdlib.h>
#include <keditcl.h>
#include <kio/netaccess.h>
#include "insert_file.h"
const int charcount = 18;
const QChar charset_org[charcount]= {353,263,281,322,324,243,156,380,159,317,262,280,321,323,211,140,379,143};
const QChar charset_new[charcount]= {261,263,281,322,324,243,347,380,378,260,262,280,321,323,211,346,379,377};
                                                       //                                                                             

enum { Local = 0, Unicode = 1, Lat1 = 2, MBug = 3 };
                                                       
KsubeditorApp::KsubeditorApp():KMainWindow(0,"Ksubeditor")
{
//// Setting config
  KsubeditorConfig = new configuration();
  configDialog = new configure(this,"Config",true);
//setting encoding
    int n = KsubeditorConfig->codecList->count();
    configDialog->encoding->clear();
    QString local = "Local (";
        local += QTextCodec::codecForLocale()->name();
        local += ")";
    configDialog->encoding->insertItem(local);
    configDialog->encoding->insertItem("Unicode");
    configDialog->encoding->insertItem("Latin");
    configDialog->encoding->insertItem("Microsoft Unicode");
     int i;
     for ( i = 4; i < n; i++ )
            configDialog->encoding->insertItem( KsubeditorConfig->codecList->at(i-4)->name());
///end of config      
  keyboard = new KAccel( this );
  config=kapp->config();
  printer = new QPrinter;
  untitledCount=0;
  pDocList = new QList<KsubeditorDoc>();
  pDocList->setAutoDelete(true);

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initView();
  initActions();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileSave->setEnabled(false);
  fileSaveAs->setEnabled(false);
  filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);


}

KsubeditorApp::~KsubeditorApp()
{
  delete printer;
}

void KsubeditorApp::initActions()
{
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

  windowNewWindow = new KAction(i18n("New &Window"), 0, this, SLOT(slotWindowNewWindow()), actionCollection(),"window_new_window");
  windowTile = new KAction(i18n("&Tile"), 0, this, SLOT(slotWindowTile()), actionCollection(),"window_tile");
  windowCascade = new KAction(i18n("&Cascade"), 0, this, SLOT(slotWindowCascade()), actionCollection(),"window_cascade");

  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));

  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));

  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

  windowMenu = new KActionMenu(i18n("&Window"), actionCollection(), "window_menu");
  connect(windowMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(windowMenuAboutToShow()));

  createGUI();

  sub_menu = new QPopupMenu();
  sub_menu->insertItem(SmallIcon("xclock"),i18n("&Move time"),this, SLOT(slotMoveSubtitles()));
  sub_menu->insertItem(SmallIcon("video"),i18n("&Change FPS"),this, SLOT(slotChangeFPS()));
  sub_menu->insertItem(SmallIcon("gear"),i18n("&Change subtitles format"),this, SLOT(slotChangeFormat()));
  sub_menu->insertItem(SmallIcon("charset"),i18n("&Convert polish chars"),this, SLOT(slotConvertCharset()));
  sub_menu->insertItem(SmallIcon("player_play"),i18n("&Test in mplayer"),this, SLOT(slotConvertCharset()));
  sub_menu->insertItem(SmallIcon("revert"),i18n("&Insert file"),this, SLOT(slotInsertFile()));
  sub_menu->insertItem(SmallIcon("view_top_bottom"),i18n("&Split to files"),this, SLOT(slotSplit()));        
  menuBar()->insertItem(i18n("&Subtitles"), sub_menu,0,2);

  options_menu = new QPopupMenu();
  menuBar()->insertItem(i18n("&Options"), options_menu,0,3);
  options_menu->insertItem(SmallIcon("keyboard"),i18n("&Keyboard config"),this, SLOT(slotKeyConfig()));
  options_menu->insertItem(SmallIcon("configure"),i18n("&Ksubeditor config"),this, SLOT(slotConfig()));

  toolBar()->insertSeparator();
  toolBar()->insertButton(BarIcon("charset"), ID_CHARSET,SIGNAL(clicked()), this, SLOT(slotConvertCharset()),true, i18n("Convert Polish letters from windows into ISO-8559-2") );
  toolBar()->insertButton(BarIcon("xclock"), ID_MOVE_TIME,SIGNAL(clicked()), this, SLOT(slotMoveSubtitles()),true, i18n("Move subtitles time") );
  toolBar()->insertButton(BarIcon("player_play"), ID_PLAY,SIGNAL(clicked()), this, SLOT(slotPlay()),true, i18n("Test in mplayer") );


 keyboard->insert( "Insert", i18n("Insert empty row under selected one."),
                       i18n("Insert empty row under selected one."),
                       Qt::Key_Insert, this, SLOT(slotInsert()) );
 keyboard->insert( "Delete", i18n("Delete selected row"),
                       i18n("Delete selected row."),
                       Qt::Key_Delete, this, SLOT(slotDelete()) );
 keyboard->insert( "Plus", i18n("Add one second to all subtitles below selected."),
                       i18n("Add one second to all subtitles below selected."),
                       Qt::Key_Equal, this, SLOT(slotPlusAll()) );
 keyboard->insert( "Minus", i18n("Back one second to all subtitles below selected."),
                       i18n("Subtract one second to all subtitles below selected."),
                       Qt::Key_Minus, this, SLOT(slotMinusAll()) );
 keyboard->insert( "Play", i18n("Start player."),
                       i18n("Start player to test subtitles."),
                       Qt::ALT+Key_P, this, SLOT(slotPlay()) );
 keyboard->insert( "Move_Line", i18n("Add one second to the current line."),
                       i18n("Add one second to the current line."),
                       Qt::Key_Plus, this, SLOT(slotPlusOne()) );
 keyboard->insert( "Move_Line2", i18n("Back one second to the current line."),
                       i18n("Back one second to the current line."),
                       Qt::Key_Underscore, this, SLOT(slotMinusOne()) );
 keyboard->insert( "Plus_Ms", i18n("Plus miliseconds (or frames) to current line."),
                       i18n("Plus miliseconds (or frames) to current line."),
                       Qt::ALT+Key_PageUp, this, SLOT(slotPlusMilisecond()) );
 keyboard->insert( "Minus_Ms", i18n("Minus miliseconds (or frames) to current line."),
                       i18n("Minus miliseconds (or frames) to current line."),
                       Qt::ALT+Key_PageDown, this, SLOT(slotMinusMilisecond()) );
 keyboard->insert( "Plus_MsAll", i18n("Plus miliseconds (or frames) to all subtitles below selected."),
                       i18n("Plus miliseconds (or frames) to all subtitles below selected."),
                       Qt::CTRL+Key_PageUp, this, SLOT(slotPlusMilisecondAll()) );
 keyboard->insert( "Minus_MsAll", i18n("Minus miliseconds (or frames) to all subtitles below selected."),
                       i18n("Minus miliseconds (or frames) to all subtitles below selected."),
                       Qt::CTRL+Key_PageDown, this, SLOT(slotMinusMilisecondAll()) );
                       
 // KKeyDialog::configure(keyboard);

}


void KsubeditorApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."),1);
  statusBar()->insertItem(i18n("        "),2);
  statusBar()->insertItem(i18n("Subtitles FPS:"),3);
}


void KsubeditorApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // here the main view of the KMainWindow is created by a background box and
  // the QWorkspace instance for MDI view.
  QVBox* view_back = new QVBox( this );
  view_back->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
  pWorkspace = new QWorkspace( view_back );
  setCentralWidget(view_back);
}


void KsubeditorApp::createClient(KsubeditorDoc* doc)
{
KsubeditorView* w = new KsubeditorView(doc, pWorkspace,0,WDestructiveClose);
w->installEventFilter(this);
  doc->addView(w);
  w->setIcon(kapp->miniIcon());
  if ( pWorkspace->windowList().isEmpty() ) // show the very first window in maximized mode
    w->showMaximized();
  else
  w->show();
    
 connect(w->ListView1, SIGNAL( contextMenuRequested( QListViewItem *, const QPoint& , int ) ),
            this, SLOT( slotRightClick(QListViewItem *, const QPoint &, int ) ) );
 connect(w->ListView1, SIGNAL( doubleClicked( QListViewItem *, const QPoint& , int ) ),
            this, SLOT( slotEdit(QListViewItem *, const QPoint &, int ) ) );
 connect(w->ListView1, SIGNAL( returnPressed( QListViewItem *) ),
            this, SLOT( slotMultiLineEdit(QListViewItem *) ) ); 
 connect(w->ListView1, SIGNAL( itemRenamed( QListViewItem *, int ) ),
            this, SLOT( slotModified() ) );
 connect(w, SIGNAL(got_focus()), this, SLOT(slotStatusBar()));            
    
}

void KsubeditorApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
  KsubeditorDoc* doc;
	// check, if document already open. If yes, set the focus to the first view
  for(doc=pDocList->first(); doc > 0; doc=pDocList->next())
  {
    if(doc->URL()==url)
    {
      KsubeditorView* view=doc->firstView();
      view->setFocus();
      return;
    }
  }
  doc = new KsubeditorDoc();
  pDocList->append(doc);
  doc->newDocument();
  fileSaveAs->setEnabled(true);
  // Creates an untitled window if file is 0	
  if(url.isEmpty())
  {
    untitledCount+=1;
    QString fileName=QString(i18n("Untitled%1")).arg(untitledCount);
    KURL url;
    url.setFileName(fileName);
    doc->setURL(url);
    createClient(doc);
    doc->openDocument(url,-1);
  }
  // Open the file
  else
  {
   QString scie;
   scie = url.url();
   KIO::NetAccess::download( url, scie );
   int code = doc->typeDetect(url.path().latin1());
   if ( code>=0){ //if not error
     createClient(doc);
          if(!doc->openDocument(url,doc->typeDetect(url.path().latin1())))
        {
          KMessageBox::error (this,i18n("Could not open document !"), i18n("Error !"));
          return;
        }
    	  fileOpenRecent->addURL(url);
   }  else {
         switch (code) {
         case SUB_INVALID: KMessageBox::error (this,i18n("Invalid subtitles format !"), i18n("Error !"));
                           return;
                           break;
         case READ_ERROR:  KMessageBox::error (this,i18n("Bad file or no read permissions !"), i18n("Error !"));
                           return;
                           break;
         }
    }
  }  
  // create the window


  slotStatusMsg(i18n("Ready."));
 if (!doc->sub_uses_time) {
  char txt[40];

  sprintf(txt,"%f",doc->subtitle->micro->fps);
  statusBar()->changeItem(i18n("Subtitles FPS: ")+txt,3);
 } else statusBar()->changeItem(i18n("Subtitles FPS: Not available"),3);
}


void KsubeditorApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", toolBar()->isVisible());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  config->writeEntry("Path to Mplayer",KsubeditorConfig->MplayerPath);
  config->writeEntry("Default font encoding",KsubeditorConfig->defaultEncoding);
  config->writeEntry("Miliseconds to move in SRT",KsubeditorConfig->move_SRT);
  config->writeEntry("Frames to move in MicroDVD",KsubeditorConfig->move_MicroDVD);
  
  fileOpenRecent->saveEntries(config,"Recent Files");
  keyboard->writeSettings();

}


void KsubeditorApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);


  KsubeditorConfig->MplayerPath = config->readPathEntry("Path to Mplayer");
  KsubeditorConfig->defaultEncoding = config->readNumEntry("Default font encoding");
  KsubeditorConfig->move_SRT = config->readNumEntry("Miliseconds to move in SRT");  
  KsubeditorConfig->move_MicroDVD = config->readNumEntry("Frames to move in MicroDVD");  
  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
  keyboard->readSettings();
}

void KsubeditorApp::saveProperties(KConfig *_cfg)
{

}


void KsubeditorApp::readProperties(KConfig* _cfg)
{
}

bool KsubeditorApp::queryClose()
{

  QStringList saveFiles;
  KsubeditorDoc* doc;
  if(pDocList->isEmpty())
    return true;

  for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
  {
    if(doc->isModified())
      saveFiles.append(doc->URL().fileName());
  }
  if(saveFiles.isEmpty())
    return true;

  switch (KMessageBox::questionYesNoList(this,
          i18n("One or more documents have been modified.\nSave changes before exiting?"),saveFiles))
  {
  case KMessageBox::Yes:
    for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
    {
      if(doc->URL().fileName().contains(i18n("Untitled")))
        slotFileSaveAs();
      else
      {
        if(!doc->saveDocument(doc->URL()))
        {
          KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
          return false;
         }
       }
     }
     return true;
  case KMessageBox::No:
  default:
  return true;
  }
}

bool KsubeditorApp::queryExit()
{
  saveOptions();
  return true;
}

bool KsubeditorApp::eventFilter(QObject* object, QEvent* event)
{
  if((event->type() == QEvent::Close)&&((KsubeditorApp*)object!=this))
  {
    QCloseEvent* e=(QCloseEvent*)event;

    KsubeditorView* pView=(KsubeditorView*)object;
    KsubeditorDoc* pDoc=pView->getDocument();
    if(pDoc->canCloseFrame(pView))
    {
      pDoc->removeView(pView);
      if(!pDoc->firstView()) {
        pDocList->remove(pDoc);
      e->accept();
    }
    }
    else
      e->ignore();
 if (KsubeditorView* w = (KsubeditorView*)pWorkspace->activeWindow()){
   fileSaveAs->setEnabled(false);
   fileSave->setEnabled(false);
 }
  }
  return QWidget::eventFilter( object, event );    // standard event processing
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void KsubeditorApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  openDocumentFile();

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  KURL url=KFileDialog::getOpenURL(QString::null,
      i18n("*|All files"), this, i18n("Open File..."));
  if(!url.isEmpty())
  {
    openDocumentFile(url);
    fileOpenRecent->addURL( url );
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
  	
  openDocumentFile(url);
	
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));

  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {
    KsubeditorDoc* doc = m->getDocument();
    if(doc->URL().fileName().contains(i18n("Untitled")))
      slotFileSaveAs();
    else
      if(!doc->saveDocument(doc->URL()))
    KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
    if( m )
    {
      KsubeditorDoc* doc =	m->getDocument();
      doc->setURL(url);
      if(!doc->saveDocument(url))
      {
        KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
        return;
      }
      doc->changedViewList();
//      setWndTitle(m);
	    fileOpenRecent->addURL(url);
    }	
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {
    KsubeditorDoc* doc=m->getDocument();
    doc->closeDocument();
    if (KsubeditorView* w = (KsubeditorView*)pWorkspace->activeWindow()) fileSaveAs->setEnabled(false);
  }
	
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
    m->print( printer );

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
        break;
    }
  }	
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditUndo()
{
  slotStatusMsg(i18n("Reverting last action..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//  m->undo();

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//    m->cut();	

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//    m->copy();
		
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));
	
  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
//    m->paste();
		
  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotWindowNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  KsubeditorView* m = (KsubeditorView*) pWorkspace->activeWindow();
  if ( m )
  {
    KsubeditorDoc* doc = m->getDocument();
    createClient(doc);
  }

  slotStatusMsg(i18n("Ready."));
}

void KsubeditorApp::slotWindowTile(){
  pWorkspace->tile();
}

void KsubeditorApp::slotWindowCascade(){
  pWorkspace->cascade();
}

void KsubeditorApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text,1);
}


void KsubeditorApp::windowMenuAboutToShow()
{
  windowMenu->popupMenu()->clear();
  windowMenu->insert(windowNewWindow);
  windowMenu->insert(windowCascade);
  windowMenu->insert(windowTile);

  if ( pWorkspace->windowList().isEmpty() ){
    windowNewWindow->setEnabled(false);
    windowCascade->setEnabled(false);
    windowTile->setEnabled(false);
  }
  else{
    windowNewWindow->setEnabled(true);
    windowCascade->setEnabled(true);
    windowTile->setEnabled(true);
  }
  windowMenu->popupMenu()->insertSeparator();

  QWidgetList windows = pWorkspace->windowList();
  for ( int i = 0; i < int(windows.count()); ++i )
  {
    int id = windowMenu->popupMenu()->insertItem(QString("&%1 ").arg(i+1)+windows.at(i)->caption(), this, SLOT( windowMenuActivated( int ) ) );
    windowMenu->popupMenu()->setItemParameter( id, i );
    windowMenu->popupMenu()->setItemChecked( id, pWorkspace->activeWindow() == windows.at(i) );
  }
}

void KsubeditorApp::windowMenuActivated( int id )
{
  QWidget* w = pWorkspace->windowList().at( id );
  if ( w )
    w->setFocus();
}

/** No descriptions */
void KsubeditorApp::slotMoveSubtitles(){

 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {
    KsubeditorDoc* doc = m->getDocument();
    if (doc->format!=SUB_INVALID){
      if(doc->subtitle->move_time()) {
                                    slotStatusMsg(i18n("Ready."));
                                    doc->setModified(true);
                                    fileSave->setEnabled(true);
                                    fileSaveAs->setEnabled(true);

                                    }
   else
    KMessageBox::error (this,i18n("Could not move subtitles !"), i18n("Is document opened ?!"));
  }else
    KMessageBox::error (this,i18n("Could not move subtitles !"), i18n("Is document opened ?!"));
  } else
    KMessageBox::error (this,i18n("Could not move subtitles !"), i18n("Is document opened ?!"));

}


void KsubeditorApp::slotRightClick(QListViewItem* item, const QPoint & point, int){
 int i;
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   right_button_menu = new QPopupMenu(m->ListView1);
   int DELETE = right_button_menu->insertItem(i18n("Delete row"));
   int INSERT = right_button_menu->insertItem(i18n("Insert row"));
   if (item)   i = right_button_menu->exec(point);
      else
          return;

   if (i == -1) return;
   if (i == DELETE) {
                    delete item;
                    m->getDocument()->setModified(true);
                    fileSave->setEnabled(true);
                    fileSaveAs->setEnabled(true);

                    }
   if (i == INSERT) {
                    item = new KListViewItem( m->ListView1, m->ListView1->currentItem() );
                    item->setRenameEnabled(0,true);
                    item->setRenameEnabled(1,true);
                    item->setRenameEnabled(2,true);
                    item->setRenameEnabled(3,true);
                    item->setRenameEnabled(4,true);
                    m->ListView1->insertItem(item);
                    m->getDocument()->setModified(true);
                    fileSave->setEnabled(true);
                    fileSaveAs->setEnabled(true);

                    }

}

/** No descriptions */
/** No descriptions */
void KsubeditorApp::slotChangeFPS(){

  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
  {
    KsubeditorDoc* doc = m->getDocument();

    if(doc->format == SUB_MICRODVD) {
             if (doc->subtitle->micro->ChangeFPS()){

               slotStatusMsg(i18n("Ready."));
               doc->setModified(true);
             }
    }
    else
    KMessageBox::error (this,i18n("Not framebased subtitle format !"), i18n("Could not change FPS ?!"));
  }


}
/** No descriptions */
void KsubeditorApp::slotChangeFormat(){

  conv_sub = new convert(this,"Change",true);
  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
 {
  KsubeditorDoc* doc = m->getDocument();
  if (doc->format == SUB_MICRODVD)
  {
    conv_sub->vplayer->setChecked(true);
  } else conv_sub->microDVD->setChecked(true);

  if (conv_sub->exec()) {
    openDocumentFile();
    KsubeditorView* m2 = (KsubeditorView*)pWorkspace->activeWindow();
    KsubeditorDoc* doc2 = m2->getDocument();
    doc2->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);
    if (conv_sub->microDVD->isChecked()) {
                doc2->subtitle  = new subtitles(SUB_MICRODVD,m2->ListView1);
                doc2->format = SUB_MICRODVD;
                char txt[12];
                sprintf(txt,"%f",doc2->subtitle->micro->fps);
                statusBar()->changeItem(i18n("Subtitles FPS: ")+txt,3);
                doc->subtitle->convert(SUB_MICRODVD,m2->ListView1,doc2->subtitle->fps);
    }
    
    if (conv_sub->vplayer->isChecked()) {
                doc2->subtitle  = new subtitles(SUB_VPLAYER,m2->ListView1);
                doc2->format = SUB_VPLAYER;
                doc->subtitle->convert(SUB_VPLAYER,m2->ListView1);
    }

    if (conv_sub->SRT->isChecked()) {
                doc2->subtitle  = new subtitles(SUB_SUBRIP,m2->ListView1);
                doc2->format = SUB_SUBRIP;
                doc->subtitle->convert(SUB_SUBRIP,m2->ListView1);
    }
    
  doc2->subtitle->check();
   }
  }
}/** No descriptions */
void KsubeditorApp::slotPlay(){
   QString command_line;
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
    KsubeditorDoc* doc = m->getDocument();

    if(doc->movie_file.isEmpty()){
    doc->movie_file=KFileDialog::getOpenURL(QString::null,
      i18n("*|All files"), this, i18n("Chose path to the MOVIE file"));
    }
    if(!doc->movie_file.isEmpty())
    {
      KTempFile * plik = new KTempFile();
      KURL url;
      url.setPath(plik->name());
      delete plik;
      doc->saveDocument(url);
      QListViewItem * starting_row = m->ListView1->selectedItem();
      QString seconds;
      switch (doc->format) {
        case SUB_MICRODVD: if (doc->subtitle->micro->checkLine(starting_row))
                                               seconds.setNum(starting_row->text(0).toInt()/doc->subtitle->fps-4);
                                           else seconds = "0";
                                           break;
        case SUB_VPLAYER:
                                           if (doc->subtitle->v_player->checkLine(starting_row))
                                             {
                                             char time[12];
                                             int len,a1,a2,a3;
                                             strcpy(time,starting_row->text(0).latin1());
                                             len=sscanf (time, "%d:%d:%d",&a1,&a2,&a3);
                                             seconds.setNum(a1*3600+a2*60+a3-4);
                                             } else seconds = "0";
                                            break;
        case SUB_SUBRIP:
                                           if (doc->subtitle->sub_rip->checkLine(starting_row))
                                             {
                                             char time[20];
                                             int len,a1,a2,a3;
                                             strcpy(time,starting_row->text(1).latin1());
                                             len=sscanf (time, "%d:%d:%d,",&a1,&a2,&a3);
                                             seconds.setNum(a1*3600+a2*60+a3-4);
                                             } else seconds = "0";
                                            break;

                                            
      }
      command_line = KsubeditorConfig->MplayerPath+" \"";
      command_line+=doc->movie_file.path()+"\" -sub \""+url.path()+"\"";
      command_line+=" -ss "+seconds;
      system(command_line.latin1());
    }
   }
}
/** No descriptions */
void KsubeditorApp::slotConvertCharset(){

   int col;

   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    switch (doc->format) {
      case SUB_MICRODVD: col = 2;
                         break;
      case SUB_VPLAYER : col = 1;
                         break;
      case SUB_SUBRIP : col = 3;
                         break;
                         
      default: break;
    }
//    KListViewItem  * item;
    doc->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);

       for (QListViewItem  * item = m->ListView1->firstChild();
                  item!=0;item = item->itemBelow())
       {
            for (int a=col;a<col+3;a++)
            {
              QString linia = item->text(a);
              for (int b = 0;b<linia.length();b++)
              {
                 for (int c = 0;c<charcount;c++)
                {
                  QChar l = linia[b];
                  if (l == charset_org[c]) linia[b] = charset_new[c];
                }
              }
              item->setText(a,linia);
            }
       }
   }

}
/** No descriptions */
/** No descriptions */
void KsubeditorApp::slotInsert(){
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   int a;
   if( m )
   {
       if (m->ListView1->currentItem()->itemAbove())
          a = m->ListView1->currentItem()->itemAbove()->text(0).toInt()+1;
          else a = 1;
       KListViewItem * item = new KListViewItem( m->ListView1, m->ListView1->currentItem()->itemAbove() );
       item->setRenameEnabled(0,true);
       item->setRenameEnabled(1,true);
       item->setRenameEnabled(2,true);
       item->setRenameEnabled(3,true);
       item->setRenameEnabled(4,true);
       m->ListView1->insertItem(item);
       m->getDocument()->setModified(true);
       fileSave->setEnabled(true);
       fileSaveAs->setEnabled(true);

       if (m->getDocument()->format == SUB_SUBRIP)
       {
         for (QListViewItem  * item2 = item;
             item2!=0;item2 = item2->itemBelow())
            {
              item2->setText(0,QString("%1").arg(a++,0,10));
            }   
       }  
                    
    }
}
/** No descriptions */
void KsubeditorApp::slotDelete(){
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   int a;
   QListViewItem *item3;
   if( m )
   {
     if (m->ListView1->selectedItem())
     {
          if (m->ListView1->selectedItem()->itemAbove())
            {
           a = m->ListView1->selectedItem()->itemAbove()->text(0).toInt();
           item3 = m->ListView1->selectedItem()->itemAbove();
            } 
          else a = 1;
                   QListViewItem * item = m->ListView1->selectedItem();           
                   delete item;
                   if (a==1) item3 = m->ListView1->firstChild();
                    m->getDocument()->setModified(true);
                    fileSave->setEnabled(true);
                    fileSaveAs->setEnabled(true);
       if (m->getDocument()->format == SUB_SUBRIP)
       {
         for (QListViewItem  * item2 = item3;
             item2!=0;item2 = item2->itemBelow())
            {
              item2->setText(0,QString("%1").arg(a++,0,10));
            }
        }
     }  
   }
}
/** No descriptions */
void KsubeditorApp::slotPlusAll(){
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
     KsubeditorDoc* doc = m->getDocument();
     doc->subtitle->move_time(1);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);

   }
}
/** No descriptions */
void KsubeditorApp::slotMinusAll(){
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
     KsubeditorDoc* doc = m->getDocument();
     doc->subtitle->move_time(-1);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);

   }
}
/** No descriptions */
void KsubeditorApp::slotKeyConfig(){
KKeyDialog::configure(keyboard);
}
/** No descriptions */
void KsubeditorApp::slotModified(){
   KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);
    m->getDocument()->subtitle->check();
  }

}
/** No descriptions */
void KsubeditorApp::slotEdit(QListViewItem * item, const QPoint & point, int c){
  item->startRename(c);
}
/** No descriptions */


/** No descriptions */
void KsubeditorApp::slotConfig(){


  configDialog->mplayer->setURL(KsubeditorConfig->MplayerPath);
  configDialog->encoding->setCurrentItem(KsubeditorConfig->defaultEncoding);
  configDialog->move_SRT->setValue(KsubeditorConfig->move_SRT);
  configDialog->move_MicroDVD->setValue(KsubeditorConfig->move_MicroDVD);
  if (configDialog->exec()) {
    KsubeditorConfig->MplayerPath = configDialog->mplayer->url();
    KsubeditorConfig->defaultEncoding = configDialog->encoding->currentItem();
    KsubeditorConfig->move_SRT =  configDialog->move_SRT->value();
    KsubeditorConfig->move_MicroDVD =  configDialog->move_MicroDVD->value();    
     }
}
/** No descriptions */
void KsubeditorApp::paintEvent(QPaintEvent * e){



}
/** No descriptions */
void KsubeditorApp::slotPlusOne(){
 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
   if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    doc->subtitle->move_time(1,true);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);

   }
}
/** No descriptions */
void KsubeditorApp::slotMinusOne(){
 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    doc->subtitle->move_time(-1,true);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);

   }
}
/** No descriptions */
void KsubeditorApp::slotStatusBar(){
 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
   {
     if (m->getDocument()->format==SUB_MICRODVD)
       {
        char txt[12];
        sprintf(txt,"%f",m->getDocument()->subtitle->micro->fps);
        statusBar()->changeItem(i18n("Subtitles FPS: ")+txt,3);     
       } 
     else statusBar()->changeItem(i18n("Subtitles FPS: Not Available"),3);     
   }

}

/** No descriptions */
void KsubeditorApp::slotPlusMilisecond(){
KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    int move;
    if (doc->format==SUB_MICRODVD) move = KsubeditorConfig->move_MicroDVD;
    if (doc->format==SUB_SUBRIP) move = KsubeditorConfig->move_SRT;

    doc->subtitle->move_ms(move,true);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);
   }
}

void KsubeditorApp::slotMinusMilisecond(){
KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    int move;
    if (doc->format==SUB_MICRODVD) move = KsubeditorConfig->move_MicroDVD;
    if (doc->format==SUB_SUBRIP) move = KsubeditorConfig->move_SRT;
    doc->subtitle->move_ms(-move,true);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);
   }
}
/** No descriptions */
void KsubeditorApp::slotPlusMilisecondAll(){
KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    int move;
    if (doc->format==SUB_MICRODVD) move = KsubeditorConfig->move_MicroDVD;
    if (doc->format==SUB_SUBRIP) move = KsubeditorConfig->move_SRT;

    doc->subtitle->move_ms(move,false);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);
   }

}
/** No descriptions */
void KsubeditorApp::slotMinusMilisecondAll(){
  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
   {
    KsubeditorDoc* doc = m->getDocument();
    int move;
    if (doc->format==SUB_MICRODVD) move = KsubeditorConfig->move_MicroDVD;
    if (doc->format==SUB_SUBRIP) move = KsubeditorConfig->move_SRT;
    doc->subtitle->move_ms(-move,false);
    m->getDocument()->setModified(true);
    fileSave->setEnabled(true);
    fileSaveAs->setEnabled(true);
   }

}
/** No descriptions */
void KsubeditorApp::slotMultiLineEdit(QListViewItem * item){
  editLine = new edit(this,"Edit");
  int start_col;
  KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
  if( m )
   {
    switch (m->getDocument()->format)
    {
      case SUB_MICRODVD:  start_col = 2;
                          break;
      case SUB_VPLAYER:   start_col = 1;
                          break;
      case SUB_SUBRIP:    start_col = 3;
                          break;
      default: start_col = 0;
               break;
    }
    editLine->lineEdit->insertLine(item->text(start_col),0);
    editLine->lineEdit->insertLine(item->text(start_col+1),1);
    editLine->lineEdit->insertLine(item->text(start_col+2),2);    
    if (editLine->exec())
    {
      item->setText(start_col,editLine->lineEdit->textLine(0));
      item->setText(start_col+1,editLine->lineEdit->textLine(1));
      item->setText(start_col+2,editLine->lineEdit->textLine(2));      
    };
   }
}
/** No descriptions */
void KsubeditorApp::slotInsertFile(){
 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
 {
    int format = -1,format2=-1;
    KURL url=KFileDialog::getOpenURL(QString::null,
      i18n("*|All files"), this, i18n("Open File..."));

    if(!url.isEmpty())
    {
      format2 = m->getDocument()->format;
      format = m->getDocument()->typeDetect(url.path().latin1());

      if (format==format2)
      {
         switch (format)
         {
           case SUB_MICRODVD:  m->getDocument()->subtitle->micro->inserting = true;
                               m->getDocument()->subtitle->micro->load(url.path().latin1());
                               m->getDocument()->subtitle->micro->inserting = false;
                               break;
           case SUB_VPLAYER:   m->getDocument()->subtitle->v_player->inserting = true;
                               m->getDocument()->subtitle->v_player->load(url.path().latin1());
                               m->getDocument()->subtitle->v_player->inserting = false;
                               break;
           case SUB_SUBRIP:    m->getDocument()->subtitle->sub_rip->inserting = true;
                               m->getDocument()->subtitle->sub_rip->load(url.path().latin1());
                               m->getDocument()->subtitle->sub_rip->inserting = false;                               
                               break;
           default:
                    break;
         }
      } else
         KMessageBox::error (this,
          i18n("Incompatible subtitle formats !"),
           i18n("Could not join files"));  
  }
 };      
}
/** No descriptions */
void KsubeditorApp::slotSplit(){

 KsubeditorView* m = (KsubeditorView*)pWorkspace->activeWindow();
 if( m )
 {
   if(m->getDocument()->subtitle->Lista->selectedItem())
   {
        QListViewItem *item2,*item3;
        openDocumentFile();
        KsubeditorView* m2 = (KsubeditorView*)pWorkspace->activeWindow();
        KsubeditorDoc *doc = m2->getDocument();
        doc->createSubtitle(m->getDocument()->subtitle->format);

        item2 = 0;
        
        for (int a=0;a<m->getDocument()->subtitle->Lista->columns();a++)
        {
          doc->subtitle->Lista->addColumn(m->getDocument()->subtitle->Lista->columnText(a));
        }
          doc->subtitle->Lista->setSorting(-1,false);        


        for(QListViewItem *item=m->getDocument()->subtitle->Lista->selectedItem();
            item!=0;)
        {
          if (item2) item2 = new KListViewItem( doc->subtitle->Lista, item2 );
            else item2 = new KListViewItem( doc->subtitle->Lista, 0 );

          for (int b = 0;b<doc->subtitle->Lista->columns();b++)
          {
            item2->setText(b,item->text(b));
          }

          item3 = item;
          item = item->itemBelow();
          m->getDocument()->subtitle->Lista->takeItem(item3);
        }

   } else
          KMessageBox::error (this,
          i18n("You have to select a splitting line!"),
           i18n("Where to split?"));

 } 

}
