/***************************************************************************
                          ksubeditordoc.cpp  -  description
                             -------------------
    begin                : sob sie  3 13:14:37 CEST 2002
    copyright            : (C) 2002 by Anatol Ogrek
    email                : mroova@linux.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <klistview.h>
// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/job.h>
#include <kio/netaccess.h>

// application specific includes
#include "ksubeditor.h"
#include "ksubeditorview.h"
#include "ksubeditordoc.h"
//#include "lista.h"

KsubeditorDoc::KsubeditorDoc()
{

  pViewList = new QList<KsubeditorView>;
  pViewList->setAutoDelete(false);
  sub_uses_time = 1;

}

KsubeditorDoc::~KsubeditorDoc()
{
  delete pViewList;
//  delete subtitle;
}

void KsubeditorDoc::addView(KsubeditorView *view)
{
  pViewList->append(view);
	changedViewList();
}

void KsubeditorDoc::removeView(KsubeditorView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void KsubeditorDoc::changedViewList(){	
	
	KsubeditorView *w;
	if((int)pViewList->count() == 1){
  	w=pViewList->first();
  	w->setCaption(doc_url.fileName());
	}
	else{	
		int i;
    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
  		w->setCaption(QString(URL().fileName()+":%1").arg(i));	
	}
}

bool KsubeditorDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void KsubeditorDoc::updateAllViews(KsubeditorView *sender)
{
  KsubeditorView *w;

  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void KsubeditorDoc::setURL(const KURL & url)
{
  doc_url=url;
}

const KURL & KsubeditorDoc::URL() const
{
  return doc_url;
}

void KsubeditorDoc::closeDocument()
{
  KsubeditorView *w;
  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
  	w=pViewList->first();
  	w->close();
  }
}

bool KsubeditorDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  format = -1;
  return true;
}

bool KsubeditorDoc::openDocument(const KURL &url, const  int form /*=0*/)
{

  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile );
  format = form;
  /////////////////////////////////////////////////
	QFile f( tmpfile );
	if ( !f.open( IO_ReadOnly ) )
		return false;
  KsubeditorView *w;
  w=pViewList->first();
  w->setCaption(url.filename());
  subtitle = new subtitles(format,w->ListView1);

  if (!(subtitle->load(tmpfile))) KMessageBox::error (w,i18n("Unsupported subtitles format !"), i18n("Error !"));;
	f.close();

  /////////////////////////////////////////////////
  KIO::NetAccess::removeTempFile( tmpfile );
  doc_url=url;
	
  modified=false;
  return true;
}

bool KsubeditorDoc::saveDocument(const KURL &url, const int format /*=0*/)
{
  KsubeditorView *w;
  w=pViewList->first();

  if (subtitle->save(url.path())) setModified(false);
//	QFile f( filename );
//	if ( !f.open( IO_WriteOnly ) )
//		return false;
//
//  /////////////////////////////////////////////////
//  // TODO: Add your document saving code here
//  /////////////////////////////////////////////////
//
//  f.close();
//
//  modified=false;
//	m_filename=filename;
//	m_title=QFileInfo(f).fileName();
  return true;
}

void KsubeditorDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool KsubeditorDoc::canCloseFrame(KsubeditorView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		KURL saveURL;
  	switch(KMessageBox::warningYesNoCancel(pFrame, i18n("The current file has been modified.\n"
                          "Do you want to save it?"),URL().fileName()))
    {
			case KMessageBox::Yes:
				if(URL().fileName().contains(i18n("Untitled")))
				{
					saveURL=KFileDialog::getSaveURL(QDir::currentDirPath(),
                      i18n("*|All files"), pFrame, i18n("Save as..."));
          if(saveURL.isEmpty())
          	return false;
				}
				else
						saveURL=URL();
					
				if(!saveDocument(saveURL))
				{
 					switch(KMessageBox::warningYesNo(pFrame,i18n("Could not save the current document !\n"
																												"Close anyway ?"), i18n("I/O Error !")))
 					{
 						case KMessageBox::Yes:
 							ret=true;
 						case KMessageBox::No:
 							ret=false;
 					}	        			
				}
				else
					ret=true;
				break;
			case KMessageBox::No:
				ret=true;
				break;
			case KMessageBox::Cancel:
			default:
				ret=false; 				
				break;
		}
	}
	else
		ret=true;
		
	return ret;
}
/** No descriptions */
int KsubeditorDoc::typeDetect(const char * fname){

    char line[LINE_LEN+1];
    int i,j=0;
    char p;
    FILE * fd;
    sub_uses_time = 1;
    fd=fopen (fname, "r"); if (!fd) return -2;        //read error
    while (j < 10) {
	j++;
	if (!fgets (line, LINE_LEN, fd))
	 { fclose(fd);  format = SUB_INVALID;  return SUB_INVALID;}

	if (sscanf (line, "{%d}{%d}", &i, &i)==2)
		{sub_uses_time=0; fclose(fd); format = SUB_MICRODVD;return SUB_MICRODVD;}
	if (sscanf (line, "{%d}{}", &i)==1)
		{sub_uses_time=0; fclose(fd); format = SUB_MICRODVD;return SUB_MICRODVD;}
	if (sscanf (line, "%d:%d:%d.%d,%d:%d:%d.%d",     &i, &i, &i, &i, &i, &i, &i, &i)==8)
		{sub_uses_time=1; fclose(fd); format = SUB_SUBVIEWER;return SUB_SUBVIEWER;}
	if (sscanf (line, "%d:%d:%d,%d --> %d:%d:%d,%d", &i, &i, &i, &i, &i, &i, &i, &i)==8)
		{sub_uses_time=1; fclose(fd); format = SUB_SUBRIP;return SUB_SUBRIP;}
	if (strstr (line, "<SAMI>"))
		{sub_uses_time=1; fclose(fd);  format = SUB_SAMI;return SUB_SAMI;}
	if (sscanf (line, "%d:%d:%d:",     &i, &i, &i )==3)
		{sub_uses_time=1; fclose(fd); format = SUB_VPLAYER;return SUB_VPLAYER;}
	if (sscanf (line, "%d:%d:%d ",     &i, &i, &i )==3)
		{sub_uses_time=1; fclose(fd); format = SUB_VPLAYER;return SUB_VPLAYER;}
	//TODO: just checking if first line of sub starts with "<" is WAY
	// too weak test for RT
	// Please someone who knows the format of RT... FIX IT!!!
	// It may conflict with other sub formats in the future (actually it doesn't)
	if ( *line == '<' )
		{sub_uses_time=1; fclose(fd); format = SUB_RT;return SUB_RT;}

	if (!memcmp(line, "Dialogue: Marked", 16))
		{sub_uses_time=1; fclose(fd);  format = SUB_SSA;return SUB_SSA;}
	if (sscanf (line, "%d,%d,\"%c", &i, &i, (char *) &i) == 3)
		{sub_uses_time=0; fclose(fd); format = SUB_DUNNOWHAT;return SUB_DUNNOWHAT;}
	if (sscanf (line, "FORMAT=%d", &i) == 1)
		{sub_uses_time=0; fclose(fd);  format = SUB_MPSUB;return SUB_MPSUB;}
	if (sscanf (line, "FORMAT=TIM%c", &p)==1 && p=='E')
		{sub_uses_time=1; fclose(fd);  format = SUB_MPSUB;return SUB_MPSUB;}
	if (strstr (line, "-->>"))
		{sub_uses_time=0; fclose(fd); format = SUB_MPSUB;return SUB_MPSUB;}
    }
    fclose(fd);
    format = SUB_INVALID;
    return SUB_INVALID;  // too many bad lines


}
/** Constructor for the fileclass of the application */
 KsubeditorDoc::KsubeditorDoc(KListView * list){

  pViewList = new QList<KsubeditorView>;
  pViewList->setAutoDelete(false);
  sub_uses_time = 1;

}
/** No descriptions */
void KsubeditorDoc::createSubtitle(int form){

  KsubeditorView *w;
  w=pViewList->first();
  subtitle = new subtitles(form,w->ListView1);
  format = form;
}
